{
#Example project_eye.com:
#
$texture_width = 1024;
$texture_height = 256;
#----------
#
# Read in the nodes for the femur
gfx read nodes $example/choroid2_fitted_fitted.exnode
gfx read elements $example/choroid2_fitted_fitted.exelem
#
# Open a graphics window.
gfx create window 1
#
# The femur is automatically shown at the centre of the window.
# Display it with a nice material
gfx create material bone ambient 0.7 0.7 0.3 diffuse 0.9 0.9 0.8 emission 0 0 0 specular 0.05 0.05 0.05 alpha 1 shininess 0.1
gfx modify window 1 set perturb_lines
#
# Next we read in a image that we are going to map onto our object.
#
gfx create texture eye_photo image $example/b1eye01.jpg
# Linearly interpolating our texture means that the texture doesn't appear so pixelated.
gfx modify texture eye_photo linear edge_clamp;
gfx mod win 1 layout width 800 height 600
gfx mod win 1 back texture eye_photo tex_placement 20 -20 760 560
gfx modify window 1 view parallel eye_point 302.649 336.917 -154.938 interest_point 307.491 265.347 -118.232 up_vector 0.587983 -0.337277 -0.735201 view_angle 40 near_clipping_plane 0.805796 far_clipping_plane 287.964 relative_viewport ndc_placement -1 1 2 2 viewport_coordinates 0 0 1 1;

#Define the surface that we want to texture

gfx create egroup iris;

#Just the elements that we want to texture
gfx modify egroup iris add elements 257..288;


#
# ---------- Interactive texturing ----------
#
# We can view the surfaces textured interactively by transforming the bone
# coordinates into 2D coordinates in the background of the window,
# and then using these coordinates to sample the texture. This field now
# contains the colour as data and we can render it with a spectrum
# that maps the three data components directly to rgb colour.
gfx define field projection window_projection field coordinates window 1 texture_projection
gfx define field mapped_texture sample_texture coordinate projection texture eye_photo;
gfx create spectrum rgb_spectrum clear overwrite_colour
gfx modify spectrum rgb_spectrum linear range 0 1 extend_above extend_below red colour_range 0 1 ambient diffuse component 1
gfx modify spectrum rgb_spectrum linear range 0 1 extend_above extend_below green colour_range 0 1 ambient diffuse component 2
gfx modify spectrum rgb_spectrum linear range 0 1 extend_above extend_below blue colour_range 0 1 ambient diffuse component 3

gfx modify g_element choroid2_fitted surfaces exterior material bone data mapped_texture spectrum rgb_spectrum;
gfx modify g_element iris surfaces as iris_surface exterior material bone data mapped_texture spectrum rgb_spectrum;
#
gfx modify g_element choroid2_fitted general element_discretization "4*4*4";
gfx modify g_element choroid2_fitted general element_discretization "12*12*12";
#gfx modify g_element choroid2_fitted lines delete;
#
# -------------- Texture coordinate space --------------
#
#
# As we are 2D texturing 3D elements we need to reorder our cooridinates so that 
# over the elements we are interested in the first and second coordinate vary from 0 to 1
# and the third coordinate is 0.
# We use computed fields to scale, offset, and reorder the integrated xi coordinates
# to ensure this.
#

gfx define field xi_texture_coordinatesA integration coordinate xi seed_element 257;

# Use the scale factors to reduce the coordinates to between 0 and 1.
# The mesh is 8 elements around, 3 deep and 2 in the surface,
# so the scale factors are 1/8, 1/3, 1/2
gfx define field xi_texture_coordinatesB scale field xi_texture_coordinatesA scale_factors 0.125 0.03333333 0.5;
# The integration field automatically grows in both directions from the seed_element,
# so start out in xi1 going from -4 to +4 which scales to -0.5 to 0.5.  Offset this back to 0
# The surface we want to texture is xi2=0 so we do not have to scale it.  If we were trying to texture another surface then we may have to offset the field so that the eventual third coordinate is 0 in the required surface.
gfx define field xi_texture_coordinatesC offset field xi_texture_coordinatesB offsets 0.5 0.0 0.0;#
gfx define field xi_texture_coordinates composite xi_texture_coordinatesC.1 xi_texture_coordinatesC.3 xi_texture_coordinatesC.2;

# To visualise the texture space we will create a scene and window
# that shows the object with the texture_coordinates chosen as the
# actual coordinates.  As above this approximation of the texture will
# update automatically as you rotate the femur in graphics window 1.
#
gfx create scene texture_projection manual;
gfx draw scene texture_projection group iris;
gfx modify g_element iris general default_coordinate xi_texture_coordinates scene texture_projection;
#gfx modify g_element iris lines scene texture_projection;
#  The normals on the surface are wrong so use an isosurface instead
#gfx modify g_element iris surfaces material bone scene texture_projection data mapped_texture spectrum rgb_spectrum;
gfx define field xi_texture_coordinates.3 component xi_texture_coordinates.3;
gfx modify g_element iris iso_surfaces as iris_surface iso_scalar xi_texture_coordinates.3 iso_values 0.0 use_elements select_on material default data mapped_texture spectrum rgb_spectrum selected_material default_selected render_shaded scene texture_projection;
gfx create window texture_projection;
gfx modify window texture_projection image scene texture_projection;
gfx modify window texture_projection layout 2d ortho_axes z -y eye_spacing 0.25 width 400 height 400;
gfx modify window texture_projection view parallel eye_point 0.5 0.5 2.88485 interest_point 0.5 0.5 0 up_vector 0 -1 -0 view_angle 29.9374 near_clipping_plane 0.0288485 far_clipping_plane 10.3095 relative_viewport ndc_placement -1 1 2 2 viewport_coordinates 0 0 1 1;

# Create the texture once so we can rerun the make_texture function.
gfx create texture eye_photo_mapped;

sub make_interactive
 {
	gfx modify g_element iris surfaces as iris_surface material default data mapped_texture spectrum rgb_spectrum;
	gfx modify g_element iris iso_surfaces as iris_surface material default scene texture_projection data mapped_texture spectrum rgb_spectrum;
 }

sub make_texture
 {
#
# -------------- High resolution texture generation --------------
#
# Now we will create a high resolution texture representation of the same data.
# This will take a while to calculate initially but once done renders almost instantly,
# however it doesn't update when you transform the window.  The maximum size of the texture
# that you can use is limited by the hardware, although you should be able to calculate one
# of an arbitrary size here.  Also OpenGL restricts you to dimensions that are multiples of 2.
#

gfx modify texture eye_photo_mapped width 1 height 1 specify_width $texture_width specify_height $texture_height evaluate_image field mapped_texture spectrum rgb_spectrum texture_coord xi_texture_coordinates element_group iris element_dimension 3;

#
# Show the texture on the bone in both windows.  The bone becomes basically invisible in 
# graphics_window 1 showing that the texturing operation was correct.  Rotate the bone to
# prove that it is actually still there.
#
gfx create material eye_photo_mapped texture eye_photo_mapped diffuse 1 1 1 ambient 0.3 0.3 0.3;
gfx modify g_element iris surfaces as iris_surface material eye_photo_mapped texture xi_texture_coordinates data none;
gfx modify g_element iris iso_surfaces as iris_surface material eye_photo_mapped texture xi_texture_coordinates scene texture_projection data none;
#
gfx write texture eye_photo_mapped file eye_photo_mapped.tiff;
#
# A linear texture mapping is slower but is less obviously pixelated
gfx modify texture eye_photo_mapped linear;
# You can change the rendition of the texture so that the bone is still shaded.
gfx modify texture eye_photo_mapped modulate;

#Hide the rest of the choroid
gfx set vis choroid2_fitted off;
#
}

sub run_this
 {
	make_interactive;
	make_texture;
 }
}
