{
#Example project_eye.com:
#
$texture_width = 64;
$texture_height = 64;
#----------
#
# Read in the nodes for the femur
gfx read nodes $example/choroid2_fitted_fitted.exnode
gfx read elements $example/choroid2_fitted_fitted.exelem
#
# Open a graphics window.
gfx create window 1
#
# The femur is automatically shown at the centre of the window.
# Display it with a nice material
gfx create material bone ambient 0.7 0.7 0.3 diffuse 0.9 0.9 0.8 emission 0 0 0 specular 0.05 0.05 0.05 alpha 1 shininess 0.1
gfx modify window 1 set perturb_lines
gfx modify g_element choroid2_fitted surfaces material bone
#
# Next we read in a image that we are going to map onto our object.
#
gfx create texture eye_photo image $example/b1eye01.jpg
# Linearly interpolating our texture means that the texture doesn't appear so pixelated.
gfx modify texture eye_photo linear edge_clamp;
gfx mod win 1 layout width 800 height 600
gfx mod win 1 back texture eye_photo tex_placement 20 -20 760 560
gfx modify window 1 view parallel eye_point 302.649 336.917 -154.938 interest_point 307.491 265.347 -118.232 up_vector 0.587983 -0.337277 -0.735201 view_angle 40 near_clipping_plane 0.805796 far_clipping_plane 287.964 relative_viewport ndc_placement -1 1 2 2 viewport_coordinates 0 0 1 1;
#
# ---------- Interactive texturing ----------
#
# We can view the surfaces textured interactively by transforming the bone
# coordinates into 2D coordinates in the background of the window,
# and then using these coordinates to sample the texture. This field now
# contains the colour as data and we can render it with a spectrum
# that maps the three data components directly to rgb colour.
gfx define field projection window_projection field coordinates window 1 texture_projection
gfx define field mapped_texture sample_texture coordinate projection texture eye_photo;
gfx create spectrum rgb_spectrum clear overwrite_colour
gfx modify spectrum rgb_spectrum linear range 0 1 extend_above extend_below red colour_range 0 1 ambient diffuse component 1
gfx modify spectrum rgb_spectrum linear range 0 1 extend_above extend_below green colour_range 0 1 ambient diffuse component 2
gfx modify spectrum rgb_spectrum linear range 0 1 extend_above extend_below blue colour_range 0 1 ambient diffuse component 3
gfx modify g_element choroid2_fitted surfaces material bone data mapped_texture spectrum rgb_spectrum;
#
# As you move the object you are displaying around the window the mapped
# colour should update automatically.  The resolution is low, controlled by
# the graphical_element sampling, typically 4x4 on each element.  Between the
# sample points OpenGL Gouraud interpolates the colour smoothing the image.
# You can improve the resolution by increasing the discretisation of your
# elements.  This works but is very demanding on your graphics as you are
# increasing the number of polygons.
gfx modify g_element choroid2_fitted general element_discretization "12*12*12";
gfx modify g_element choroid2_fitted lines delete;
#
# -------------- Texture coordinate space --------------
#
# Instead of Gouroud Shading we can make a texture which gives the high resolution colour
# without the high resolution geometry.
#
# Firstly you need to choose some texture coordinates.  These should
# evenly spread your texture over your object.  The integration field
# called texture_coordinates uses the element mesh of your model to create
# texture coordinates.  Starting at the seed_element it spreads out.
# We want to offset and scale the texture coordinates so they lie from
# 0 to 1 in each direction.
#
gfx define field one composite 1.0;
gfx define field xi_texture_coordinatesA integration coordinate xi integrand one seed_element 4;
gfx define field xi_texture_coordinatesB scale field xi_texture_coordinatesA scale_factors 0.15 0.038461538 0.04;
gfx define field xi_texture_coordinatesC offset field xi_texture_coordinatesB offsets 0.5 0.0 0.0;#
gfx define field xi_texture_coordinates composite xi_texture_coordinatesC.1 xi_texture_coordinatesC.3 xi_texture_coordinatesC.2;

# To visualise the texture space we will create a scene and window
# that shows the object with the texture_coordinates chosen as the
# actual coordinates.  As above this approximation of the texture will
# update automatically as you rotate the femur in graphics window 1.
#
gfx create scene texture_projection manual;
gfx draw scene texture_projection group choroid2_fitted;
gfx modify g_element choroid2_fitted general default_coordinate xi_texture_coordinates scene texture_projection;
#gfx modify g_element choroid2_fitted lines scene texture_projection;
#  The normals on the surface are wrong so use an isosurface instead
#gfx modify g_element choroid2_fitted surfaces material bone scene texture_projection data mapped_texture spectrum rgb_spectrum;
gfx define field xi_texture_coordinates.3 component xi_texture_coordinates.3;
gfx modify g_element choroid2_fitted iso_surfaces iso_scalar xi_texture_coordinates.3 iso_values 0.0 use_elements select_on material default data mapped_texture spectrum rgb_spectrum selected_material default_selected render_shaded scene texture_projection;
gfx create window texture_projection;
gfx modify window texture_projection image scene texture_projection;
gfx modify window texture_projection layout 2d ortho_axes z y eye_spacing 0.25 width 400 height 400;
gfx modify window texture_projection view parallel eye_point 0.5 0.36 2.85469 interest_point 0.5 0.36 0 up_vector -0 -1 -0 view_angle 40 near_clipping_plane 0.0285469 far_clipping_plane 10.2017 relative_viewport ndc_placement -1 1 2 2 viewport_coordinates 0 0 1 1;

#
# -------------- High resolution texture generation --------------
#
# Now we will create a high resolution texture representation of the same data.
# This will take a while to calculate initially but once done renders almost instantly,
# however it doesn't update when you transform the window.  The maximum size of the texture
# that you can use is limited by the hardware, although you should be able to calculate one
# of an arbitrary size here.  Also OpenGL restricts you to dimensions that are multiples of 2.
#

# Command updated by fixcom.pl on Wed Feb 12 11:37:26 2003
# Old command: gfx create texture eye_photo_mapped width 1 height 1 evaluate_image field mapped_texture spectrum rgb_spectrum width $texture_width height $texture_height texture_coord xi_texture_coordinates element_group choroid2_fitted;
gfx create texture eye_photo_mapped width 1 height 1 specify_width $texture_width specify_height $texture_height evaluate_image field mapped_texture spectrum rgb_spectrum texture_coord xi_texture_coordinates element_group choroid2_fitted;

#
# Show the texture on the bone in both windows.  The bone becomes basically invisible in 
# graphics_window 1 showing that the texturing operation was correct.  Rotate the bone to
# prove that it is actually still there.
#
gfx create material eye_photo_mapped texture eye_photo_mapped diffuse 1 1 1 ambient 0.3 0.3 0.3;
gfx modify g_element choroid2_fitted surfaces material eye_photo_mapped texture xi_texture_coordinates;
#gfx modify g_element choroid2_fitted surfaces material eye_photo_mapped texture xi_texture_coordinates scene texture_projection;
#
gfx write texture eye_photo_mapped file eye_photo_mapped.tiff;
#
# A linear texture mapping is slower but is less obviously pixelated
gfx modify texture eye_photo_mapped linear;
# You can change the rendition of the texture so that the bone is still shaded.
gfx modify texture eye_photo_mapped modulate;
#
}
